#! /usr/bin/perl
my %res = ();

while (<>) {
	chomp;
	my ($zl, $rl, $num_freq, $fft_cutoff, $benchmark, $force_16bpp,
		$truncate, $r300_hack, $time, $os, $vendor, $renderer,
		$version) = split /\t/, $_;
	
	next if ($zl != 9 || $rl != 9 || $benchmark != 1 || $force_16bpp != 0 || $truncate != 0);

	if (exists($res{$renderer}{$num_freq}{$fft_cutoff})) {
		$res{$renderer}{$num_freq}{$fft_cutoff}{total_time} += $time;
		$res{$renderer}{$num_freq}{$fft_cutoff}{num}++;
	} else {
		$res{$renderer}{$num_freq}{$fft_cutoff}{total_time} = $time;
		$res{$renderer}{$num_freq}{$fft_cutoff}{num} = 1;
	}
}

my %best = ();

for my $renderer (sort keys %res) {
	my $rd = $res{$renderer};
	printf "%-30s\n", $renderer;

	for my $num_freq (sort { $a <=> $b } (keys %$rd)) {
		my $rn = $rd->{$num_freq};

		my $best_nf = 9999999.0;
		my $best_index = 0;

		for my $cutoff (keys %$rn) {
			my $this_nf = $rn->{$cutoff}{total_time} / $rn->{$cutoff}{num};
			if ($this_nf < $best_nf) {
				$best_nf = $this_nf;
				$best_index = $cutoff;
			}
		}

		$best{$renderer}{$num_freq} = $best_nf;

		printf "  size = %3u:  %6.3f sec (optimal cutoff = %3u)\n", 
			$num_freq, $best_nf, $best_index;
	}

	print "\n";
}

# output tables we can use in TeX
print "% Raw results\n";
for my $renderer (sort { $best{$b}{512} <=> $best{$a}{512} } keys %best) {
	printf "%-40s ", $renderer;

	for my $num_freq qw(1 2 4 8 16 32 64 128 256 512) {
		if (exists($best{$renderer}{$num_freq})) {
			printf "& %6.3f ", $best{$renderer}{$num_freq};
		} else {
			printf "&  \\na   ";
		}
	}	
	print "\\\\\\hline\n";
}

my $cpu_result = 9.588;

print "\n% Relative results\n";
for my $renderer (sort { $best{$b}{512} <=> $best{$a}{512} } keys %best) {
	printf "%-40s ", $renderer;

	for my $num_freq qw(1 2 4 8 16 32 64 128 256 512) {
		$nf_use = $num_freq;
		while (!exists($best{$renderer}{$nf_use}) && $nf_use < 512) {
			$nf_use *= 2;
		}

		if (exists($best{$renderer}{$nf_use})) {
			printf "& %6.1f ", ($cpu_result * $num_freq) / ($best{$renderer}{$nf_use});
		} else {
			printf "&   \\na   ";
		}
	}	
	print "\\\\\\hline\n";
}
