#ifndef _SHADER_H
#define _SHADER_H 1

#include <GL/gl.h>
#include <vector>
#include <string>
#include <algorithm>

#ifdef WIN32
#include "glext.h"
#endif

/*
 * A utility class to encapsulate many of the common operations
 * we will need to use in our shaders.
 *
 * Shader::Binding is a kind of cleaning-up class; just create one
 * when you want to use the shader, and when the binding object
 * goes out of scope, it will take care to unbind textures as 
 * neccessary.
 */

class Shader
{
public:
	class Binding;

	Shader(const std::string &vs_filename, const std::string &fs_filename, bool enable_r300_hack = false);
	~Shader();

private:
	void compile_shader(GLuint obj, const std::string &filename);
	void check_linked();

	GLuint prog;
	bool enable_r300_hack;
};
	
class Shader::Binding
{
	friend class Shader;

private:
	const Shader &s;
	unsigned current_texunit;
	std::vector<std::pair<GLenum, GLenum> > active_bindings;

public:
	Binding(Shader &s);
	~Binding();
	void bind_texture_1d(const std::string &name, GLuint tex);
	void bind_texture_2d(const std::string &name, GLuint tex);
	void set_uniform(const std::string &name, GLfloat x);   // would be overloaded if we needed it
};

#endif /* !defined(_SHADER_H) */
